@@ -97,7 +97,9 @@ scsiblk;
 #define MODE_SELECT             0x15
 #define MODE_SENSE              0x1a
 #define SCAN                    0x1b
+#define IMPRINTER               0xc1
 #define HW_STATUS               0xc2
+#define RESET_UNIT              0xf1
 
 
 /* ==================================================================== */
@@ -114,6 +116,12 @@ static scsiblk release_unitB = { release_unitC, sizeof (release_unitC) };
 
 /* ==================================================================== */
 
+static unsigned char reset_unitC[] =
+  { RESET_UNIT, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
+static scsiblk reset_unitB = { reset_unitC, sizeof (reset_unitC) };
+
+/* ==================================================================== */
+
 static unsigned char inquiryC[] = { INQUIRY, 0x00, 0x00, 0x00, 0x1f, 0x00 };
 static scsiblk inquiryB = { inquiryC, sizeof (inquiryC) };
 
@@ -179,9 +187,18 @@ static scsiblk inquiryB = { inquiryC, sizeof (inquiryC) };
 #define get_IN_trancepareny(in)            getbitfield(in+0x20, 1, 5)
 #define get_IN_flatbed(in)                 getbitfield(in+0x20, 1, 6)
 #define get_IN_adf(in)                     getbitfield(in+0x20, 1, 7)
+#define get_IN_buffer_bytes(in)            getnbyte(in + 0x22, 4)
 #define get_IN_has_set_subwindow(in)       (getnbyte(in+0x2a, 2)) & 1
 #define get_IN_has_imprinter(in)           (getnbyte(in+0x2a, 2)) & 2
 #define get_IN_has_hw_status(in)           (getnbyte(in+0x2a, 2)) & 4
+#define get_IN_brightness_steps(in)        getnbyte(in+0x52, 1)
+#define get_IN_threshold_steps(in)         getnbyte(in+0x53, 1)
+#define get_IN_contrast_steps(in)          getnbyte(in+0x54, 1)
+#define get_IN_num_gamma(in)               getbitfield(in+0x57, 15, 4)
+#define get_IN_num_gamma_download(in)      getbitfield(in+0x57, 15, 0)
+#define get_IN_ipc_bw_reverse(in)          getbitfield(in+0x58, 1, 7)
+#define get_IN_ipc_auto1(in)               getbitfield(in+0x58, 1, 6)
+#define get_IN_ipc_auto2(in)               getbitfield(in+0x58, 1, 5)
 #define get_IN_ipc_outline_extraction(in)  getbitfield(in+0x58, 1, 4)
 #define get_IN_ipc_image_emphasis(in)      getbitfield(in+0x58, 1, 3)
 #define get_IN_ipc_auto_separation(in)     getbitfield(in+0x58, 1, 2)
@@ -196,6 +213,9 @@ static scsiblk inquiryB = { inquiryC, sizeof (inquiryC) };
 #define get_IN_compression_JPG_BASE(in)    getbitfield(in+0x5a, 1, 3)
 #define get_IN_compression_JPG_EXT(in)     getbitfield(in+0x5a, 1, 2)
 #define get_IN_compression_JPG_INDEP(in)   getbitfield(in+0x5a, 1, 1)
+#define get_IN_imprinter(in)               getbitfield(in+0x5c, 1, 7)
+#define get_IN_imprinter_stamp(in)         getbitfield(in+0x5c, 1, 6)
+#define get_IN_imprinter_electrical(in)    getbitfield(in+0x5c, 1, 5)
 
 /* ==================================================================== */
 
@@ -210,6 +230,16 @@ static unsigned char set_windowC[] =
   { SET_WINDOW, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, };
 /* opcode,  lun,  _____4 X reserved____,  transfer length, control byte */
 static scsiblk set_windowB = { set_windowC, sizeof (set_windowC) };
+#define set_SW_xferlen(sb, len) putnbyte(sb + 0x06, len, 3)
+
+	/* With the fi-series scanners, we have to use a 12-byte command
+	 * instead of a 10-byte command when communicating via USB.  This
+	 * may be a firmware bug. */
+static unsigned char set_usb_windowC[] =
+  { SET_WINDOW, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00 };
+/* opcode,  lun,  _____4 X reserved____,  transfer length, control byte */
+static scsiblk set_usb_windowB = { set_usb_windowC, sizeof (set_usb_windowC) };
 #define set_SW_xferlen(sb, len) putnbyte(sb + 0x06, len, 3)
 
 /* ==================================================================== */
@@ -226,18 +256,23 @@ static scsiblk object_positionB =
 
 /* ==================================================================== */
 
-/*
-  static unsigned char sendC[] =
+
+static unsigned char sendC[] =
   {SEND, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
-  static scsiblk sendB =
+static scsiblk sendB =
   {sendC, sizeof (sendC)};
-*/
+
 
 #define set_S_datatype_code(sb, val) sb[0x02] = (unsigned char)val
 #define S_datatype_imagedatai		0x00
-#define S_EX_datatype_LUT			0x01	/* Experiment code */
+#define S_datatype_halftone_mask        0x02
+#define S_datatype_gamma_function       0x03
+#define S_datatype_LUT_data             0x83
+#define S_datatype_jpg_q_table          0x88
+#define S_datatype_imprinter_data       0x90
+#define S_EX_datatype_LUT		0x01	/* Experiment code */
 #define S_EX_datatype_shading_data	0xa0	/* Experiment code */
-#define S_user_reg_gamma			0xc0
+#define S_user_reg_gamma		0xc0
 #define S_device_internal_info		0x03
 #define set_S_datatype_qual_upper(sb, val) sb[0x04] = (unsigned char)val
 #define S_DQ_none	0x00
@@ -249,6 +284,44 @@ static scsiblk object_positionB =
 #define S_DQ_Reg3	0x03
 #define set_S_xfer_length(sb, val)    putnbyte(sb + 0x06, val, 3)
 
+static unsigned char send_imprinterC[] = 
+  {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
+   0x00, 0x00};
+static scsiblk send_imprinterB = 
+  {send_imprinterC, sizeof(send_imprinterC)};
+
+/* imprinter counter
+ * 0 = increase counter
+ * 1 = decrease counter
+ */
+#define set_imprinter_cnt_dir(sb, val) setbitfield(sb + 0x01, 1, 5, val)
+#define S_im_dir_inc 0
+#define S_im_dir_dec 1
+/* counter value
+ * 1 = 24 bit
+ * 0 = 16 bit
+ */
+#define set_imprinter_lap24(sb, val) setbitfield(sb + 0x01, 1, 4, val)
+#define S_im_ctr_24bit 1
+#define S_im_ctr_16bit 0
+/* stepping of the imprinter counter
+ * 0..2 allowed
+ */
+#define set_imprinter_cstep(sb, val) setbitfield(sb + 0x01, 0x03, 0, val)
+#define set_imprinter_uly(sb, val) putnbyte(sb + 0x06, val, 4)
+
+/* specifies the way of printing direction of the strings.
+ */
+#define set_imprinter_dirs(sb, val) setbitfield(sb + 0x0c, 0x03, 0, val)
+#define S_im_dir_left_right 0
+#define S_im_dir_top_bottom 1
+#define S_im_dir_right_left 2
+#define S_im_dir_bottom_top 3
+
+#define set_imprinter_string_length(sb, len)  putnbyte(sb + 0x11, len, 1)
+#define max_imprinter_string_length 40
+
 /*
    static unsigned char gamma_user_LUT_LS1K[512] = { 0x00 };
    static scsiblk gamma_user_LUT_LS1K_LS1K = {
@@ -258,6 +331,34 @@ static scsiblk object_positionB =
 
 /* ==================================================================== */
 
+static unsigned char imprinterC[] =
+  { IMPRINTER, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
+static scsiblk imprinterB = { imprinterC, sizeof (imprinterC) };
+
+#define set_IM_xfer_length(sb, val) putnbyte(sb + 0x7, val, 2)
+
+static unsigned char imprinter_descC[] = 
+  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
+static scsiblk imprinter_descB = {imprinter_descC, sizeof(imprinter_descC) };
+
+/* enable/disable imprinter printing*/
+#define set_IMD_enable(sb, val) setbitfield(sb + 0x01, 1, 7, val)
+#define IMD_enable 0
+#define IMD_disable 1
+/* specifies thes side of a document to be printed */
+#define set_IMD_side(sb, val) setbitfield(sb + 0x01, 1, 6, val)
+#define IMD_front 0
+#define IMD_back 1
+/* format of the counter 16/24 bit*/
+#define set_IMD_format(sb, val) setbitfield(sb + 0x01, 1, 5, val)
+#define IMD_16_bit 0
+#define IMD_24_bit 1
+/* initial count */
+#define set_IMD_initial_count_16(sb, val) putnbyte(sb + 0x02, val, 2)
+#define set_IMD_initial_count_24(sb, val) putnbyte(sb + 0x03, val, 3)
+
+/* ==================================================================== */
+
 static unsigned char readC[] =
   { READ, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
 /* Type, rsvd, type qual, __xfer length__, Ctl */
@@ -289,6 +390,17 @@ static scsiblk mode_select_headerB = {
 };
 
 
+	/* With the fi-series scanners, we have to use a 10-byte header
+	 * instead of a 4-byte header when communicating via USB.  This
+	 * may be a firmware bug. */
+static unsigned char mode_select_usb_headerC[] = {
+  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
+};
+static scsiblk mode_select_usb_headerB = {
+  mode_select_usb_headerC, sizeof (mode_select_usb_headerC)
+};
+
+
 static unsigned char mode_select_parameter_blockC[] = {
   0x00, 0x00, 0x00,
   0x00, 0x00, 0x00,
@@ -364,7 +476,7 @@ static unsigned char hw_statusC[] =
   { HW_STATUS, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
 static scsiblk hw_statusB = { hw_statusC, sizeof (hw_statusC) };
 
-#define set_HW_allocation_length(sb, len) putnbyte(sb + 0x06, len, 2)
+#define set_HW_allocation_length(sb, len) putnbyte(sb + 0x07, len, 2)
 
 #define get_HW_B5_present(in)              getbitfield(in+0x02, 1, 0)
 #define get_HW_A4_present(in)              getbitfield(in+0x02, 1, 1)
@@ -561,6 +673,10 @@ static unsigned char window_descriptor_blockC[] = {
 #define WD_cmp_MR   2
 #define WD_cmp_MMR  3
 #define WD_cmp_JBIG 0x80
+#define WD_cmp_JPG1 0x81
+#define WD_cmp_JPG2 0x82
+#define WD_cmp_JPG3 0x83
+
 
   /* 0x21 - compression argument
    *        3091 - not supported, use 0x00
@@ -804,12 +920,11 @@ static scsiblk window_descriptor_blockB =
 
 /* ==================================================================== */
 
-/*
   static unsigned char request_senseC[] =
-  {REQUEST_SENSE, 0x00, 0x00, 0x00, 0x00, 0x00};
+  {REQUEST_SENSE, 0x00, 0x00, 0x00, 0x12, 0x00,
+   0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
   static scsiblk request_senseB =
   {request_senseC, sizeof (request_senseC)};
-*/
 
 #define set_RS_allocation_length(sb,val) sb[0x04] = (unsigned char)val
 /* defines for request sense return block */
@@ -823,7 +938,8 @@ static scsiblk window_descriptor_blockB =
 #define get_RS_additional_length(b)       b[0x07]	/* always 10 */
 #define get_RS_ASC(b)                     b[0x0c]
 #define get_RS_ASCQ(b)                    b[0x0d]
-#define get_RS_SKSV(b)                    getbitfield(b+0x0f,1,7)	/* valid, always 0 */
+#define get_RS_SKSV(b)                    getbitfield(b+0x0f,1,7)   /* valid, always 0 */
+#define get_RS_offending_byte(b)          getnbyte(b+0x10, 2)
 
 #define rs_return_block_size              18	/* Says Nikon */
 
