@@ -1,14 +1,9 @@
-/*.............................................................................
- * Project : SANE library for Plustek parallelport flatbed scanners.
- *.............................................................................
- */
-
 /* @file plustek-pp_p12.c
  * @brief p12 and pt12 specific stuff
  *
  * based on sources acquired from Plustek Inc.
  * Copyright (C) 2000 Plustek Inc.
- * Copyright (C) 2001-2003 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2001-2004 Gerhard Jaeger <gerhard@gjaeger.de>
  *
  * History:
  * - 0.38 - initial version
@@ -16,6 +11,7 @@
  * - 0.40 - no changes
  * - 0.41 - no changes
  * - 0.42 - removed setting of ps->sCaps.dwFlag in p12InitiateComponentModel()
+ * - 0.43 - no changes
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -69,8 +65,7 @@ static RegDef p12CcdStop[] = {
 
 /*************************** local functions *********************************/
 
-/*.............................................................................
- * init the stuff according to the buttons
+/** init the stuff according to the buttons
  */
 static void p12ButtonSetup( pScanData ps, Byte nrOfButtons )
 {
@@ -81,15 +76,13 @@ static void p12ButtonSetup( pScanData ps, Byte nrOfButtons )
 
     ps->AsicReg.RD_MotorDriverType |= _BUTTON_DISABLE;
     ps->Scan.motorPower            |= _BUTTON_DISABLE;
-    ps->sCaps.dwFlag               |= (SFLAG_MULTIFUNC | SFLAG_BUTTONOPT);
 }
 
-/*.............................................................................
- * According to what we have detected, set the other stuff
+/** According to what we have detected, set the other stuff
  */
 static void p12InitiateComponentModel( pScanData ps )
 {
-    /* preset some stuff and do te differences later */
+    /* preset some stuff and do the differences later */
     ps->Device.buttons        = 0;
     ps->Device.Model1Mono     = _BUTTON_MODE + _CCD_SHIFT_GATE + _SCAN_GRAYTYPE;
     ps->Device.Model1Color    = _BUTTON_MODE + _CCD_SHIFT_GATE;
