@@ -46,7 +46,7 @@
    This file is a component of the implementation of a backend for many
    of the the AGFA SnapScan and Acer Vuego/Prisa flatbed scanners. */
 
-/* $Id: snapscan.h,v 1.26 2003/10/21 20:43:25 oliver-guest Exp $
+/* $Id: snapscan.h,v 1.33 2005/02/08 22:17:53 oliver-guest Exp $
    SANE SnapScan backend */
 
 #ifndef snapscan_h
@@ -100,7 +100,9 @@ typedef enum
     PRISA5000,          /* Acer ScanPrisa 5000 - 1200 DPI */
     PRISA5300,          /* Acer ScanPrisa 5300 - 1200 DPI */
     PERFECTION660,      /* Epson Perfection 660 - 1200 DPI */
+    PERFECTION1270,     /* Epson Perfection 1270 - 1600 DPI */
     PERFECTION1670,     /* Epson Perfection 1670 - 1600 DPI */
+    PERFECTION2480,
     ARCUS1200		/* Agfa Arcus 1200 - 1200 DPI (rebadged Acer?) */
 } SnapScan_Model;
 
@@ -134,7 +136,9 @@ static struct SnapScan_Driver_desc drivers[] =
     {PRISA5300,      "Acer5300"},
     {ARCUS1200,      "Arcus1200"},
     {PERFECTION660,  "Perfection 660"},
-    {PERFECTION1670, "Perfection 1670"}
+    {PERFECTION1270, "Perfection 1270"},
+    {PERFECTION1670, "Perfection 1670"},
+    {PERFECTION2480, "Perfection 2480"}
 };
 
 #define known_drivers ((int) (sizeof(drivers)/sizeof(drivers[0])))
@@ -164,6 +168,8 @@ static struct SnapScan_Model_desc scanners[] =
     {"FlatbedScanner23",    PRISA4300_2},
     {"FlatbedScanner24",    PRISA5300},
     {"FlatbedScanner25",    PRISA5000},
+    {"FlatbedScanner40",    PRISA5000}, /* 5250C */
+    {"FlatbedScanner42",    PRISA5000}, /* 5000S */
     {"SNAPSCAN 1212U",      SNAPSCAN1212U},
     {"SNAPSCAN 1212U_2",    SNAPSCAN1212U},
     {"SNAPSCAN e10",        SNAPSCANE20},
@@ -183,8 +189,10 @@ static struct SnapScan_Model_desc scanners[] =
     {"ACERSCAN_A4____1",    ACER300F},
     {"Perfection 660",      PERFECTION660},
     {"EPSON Scanner",       PERFECTION1670}, /* dummy entry to detect scanner */
+    {"EPSON Scanner1",      PERFECTION2480}, /* dummy entry to detect scanner */
     {"ARCUS 1200",          ARCUS1200}
 };
+
 #define known_scanners ((int) (sizeof(scanners)/sizeof(scanners[0])))
 
 static char *vendors[] =
@@ -198,12 +206,28 @@ static char *vendors[] =
 };
 #define known_vendors ((int) (sizeof(vendors)/sizeof(vendors[0])))
 
+/* Known vendor IDs */
+#define USB_VENDOR_AGFA  0x06bd
+#define USB_VENDOR_ACER  0x04a5
+#define USB_VENDOR_EPSON 0x04b8
+
+/* Agfa product IDs */
+#define USB_PRODUCT_1212U2 0x2061
+/* Acer product IDs */
+#define USB_PRODUCT_PRISA310 0x12a0
+#define USB_PRODUCT_PRISA320 0x2022
+/* Epson product IDs */
+#define USB_PRODUCT_PERFECTION660  0x0114
+#define USB_PRODUCT_PERFECTION1270 0x0120
+#define USB_PRODUCT_PERFECTION1670 0x011f
+#define USB_PRODUCT_PERFECTION2480 0x0121
+
 static SANE_Word usb_vendor_ids[] =
 {
     /* USB Vendor IDs */
-    0x06bd,     /* Agfa */
-    0x04a5,     /* Acer */
-    0x04b8      /* Epson */
+    USB_VENDOR_AGFA,     /* Agfa */
+    USB_VENDOR_ACER,     /* Acer */
+    USB_VENDOR_EPSON      /* Epson */
 };
 #define known_usb_vendor_ids ((int) (sizeof(usb_vendor_ids)/sizeof(usb_vendor_ids[0])))
 
@@ -216,8 +240,12 @@ struct SnapScan_USB_Model_desc
 
 static struct SnapScan_USB_Model_desc usb_scanners[] =
 {
-    {0x04a5, 0x1a20, PRISA310},  /* Acer 310U */
-    {0x04a5, 0x2022, PRISA310}   /* Acer 320U */
+    {USB_VENDOR_ACER, USB_PRODUCT_PRISA310, PRISA310},  /* Acer 310U */
+    {USB_VENDOR_ACER, USB_PRODUCT_PRISA320, PRISA310},  /* Acer 320U */
+    {USB_VENDOR_EPSON, USB_PRODUCT_PERFECTION660, PERFECTION660}, /* Epson Perfection 660 */
+    {USB_VENDOR_EPSON, USB_PRODUCT_PERFECTION1270, PERFECTION1270}, /* Epson Perfection 1270 */
+    {USB_VENDOR_EPSON, USB_PRODUCT_PERFECTION1670, PERFECTION1670}, /* Epson Perfection 1670 */
+    {USB_VENDOR_EPSON, USB_PRODUCT_PERFECTION2480, PERFECTION2480} /* Epson Perfection 2480 */
 };
 #define known_usb_scanners ((int) (sizeof(usb_scanners)/sizeof(usb_scanners[0])))
 
@@ -339,6 +367,7 @@ struct snapscan_scanner
     SANE_Byte chroma_offset[3];   /* chroma offsets */
     SANE_Int chroma;
     Source *psrc;                 /* data source */
+    Source *preadersrc;                 /* data source for reader thread */
     SANE_Option_Descriptor options[NUM_OPTS];  /* the option descriptors */
     Option_Value val[NUM_OPTS];  /* the options themselves... */
     SANE_Int res;                /* resolution */
@@ -377,6 +406,27 @@ struct snapscan_scanner
 
 /*
  * $Log: snapscan.h,v $
+ * Revision 1.33  2005/02/08 22:17:53  oliver-guest
+ * Added IDs for Benq 5250C and 5000S
+ *
+ * Revision 1.32  2005/01/18 20:36:13  oliver-guest
+ * Added ID for Benq 5250C
+ *
+ * Revision 1.31  2004/12/01 22:12:04  oliver-guest
+ * Added support for Epson 1270
+ *
+ * Revision 1.30  2004/09/02 20:59:12  oliver-guest
+ * Added support for Epson 2480
+ *
+ * Revision 1.29  2004/06/16 19:52:27  oliver-guest
+ * Don't enforce even number of URB packages on 1212u_2. Fixes bug #300753.
+ *
+ * Revision 1.28  2004/04/08 21:53:10  oliver-guest
+ * Use sanei_thread in snapscan backend
+ *
+ * Revision 1.27  2004/03/22 00:00:40  oliver-guest
+ * Added detection for Epson 660 by USB ID since new models use new ID strings
+ *
  * Revision 1.26  2003/10/21 20:43:25  oliver-guest
  * Bugfixes for SnapScan backend
  *
