@@ -1,5 +1,5 @@
 /* sane - Scanner Access Now Easy.
-   Copyright (C) 2003 Rene Rebe (sanei_read_int)
+   Copyright (C) 2003 Rene Rebe (sanei_read_int,sanei_set_timeout)
    Copyright (C) 2001 - 2003 Henning Meier-Geinitz
    Copyright (C) 2001 Frank Zago (sanei_usb_control_msg)
    This file is part of the SANE package.
@@ -55,6 +55,9 @@
 #include <stdio.h>
 #include <dirent.h>
 
+#ifdef HAVE_RESMGR
+#include <resmgr.h>
+#endif
 #ifdef HAVE_LIBUSB
 #include <usb.h>
 #endif /* HAVE_LIBUSB */
@@ -83,12 +86,17 @@ typedef struct
   SANE_Int product;
   SANE_Int bulk_in_ep;
   SANE_Int bulk_out_ep;
+  SANE_Int iso_in_ep;
+  SANE_Int iso_out_ep;
   SANE_Int int_in_ep;
   SANE_Int int_out_ep;
+  SANE_Int control_in_ep;
+  SANE_Int control_out_ep;
   SANE_Int interface_nr;
 #ifdef HAVE_LIBUSB
   usb_dev_handle *libusb_handle;
   struct usb_device *libusb_device;
+
 #endif				/* HAVE_LIBUSB */
 }
 device_list_type;
@@ -130,6 +138,9 @@ struct ctrlmsg_ioctl
   void *data;
 }
 cmsg;
+#elif defined(__BEOS__)
+#include <drivers/USB_scanner.h>
+#include <kernel/OS.h>
 #endif /* __linux__ */
 
 static SANE_Bool inited = SANE_FALSE;
@@ -142,7 +153,7 @@ print_buffer (const SANE_Byte * buffer, SANE_Int size)
 {
 #define NUM_COLUMNS 16
 #define PRINT_BUFFER_SIZE (6 + NUM_COLUMNS * (3 + 1) + 1 + 1)
-  char line_str [PRINT_BUFFER_SIZE];
+  char line_str[PRINT_BUFFER_SIZE];
   char *pp;
   int column;
   int line;
@@ -157,7 +168,7 @@ print_buffer (const SANE_Byte * buffer, SANE_Int size)
       for (column = 0; column < NUM_COLUMNS; column++)
 	{
 	  if ((line * NUM_COLUMNS + column) < size)
-	    sprintf (pp, "%02X ", buffer [line * NUM_COLUMNS + column]);
+	    sprintf (pp, "%02X ", buffer[line * NUM_COLUMNS + column]);
 	  else
 	    sprintf (pp, "   ");
 	  pp += 3;
@@ -166,14 +177,14 @@ print_buffer (const SANE_Byte * buffer, SANE_Int size)
 	{
 	  if ((line * NUM_COLUMNS + column) < size)
 	    sprintf (pp, "%c",
-		     (buffer [line * NUM_COLUMNS + column] < 127) && 
-		     (buffer [line * NUM_COLUMNS + column] > 31) ?
-		     buffer [line * NUM_COLUMNS + column] : '.');
+		     (buffer[line * NUM_COLUMNS + column] < 127) &&
+		     (buffer[line * NUM_COLUMNS + column] > 31) ?
+		     buffer[line * NUM_COLUMNS + column] : '.');
 	  else
 	    sprintf (pp, ".");
 	  pp += 1;
 	}
-      DBG(11, "%s\n", line_str);
+      DBG (11, "%s\n", line_str);
     }
 }
 
@@ -194,8 +205,21 @@ kernel_get_vendor_product (int fd, int *vendorID, int *productID)
 	DBG (3, "sanei_usb_get_vendor_product: ioctl (product) "
 	     "of device %d failed: %s\n", fd, strerror (errno));
     }
-#endif /* defined (__linux__) */
-      /* put more os-dependant stuff ... */
+#elif defined(__BEOS__)
+  {
+    uint16 vendor, product;
+    if (ioctl (fd, B_SCANNER_IOCTL_VENDOR, &vendor) != B_OK)
+      DBG (3, "kernel_get_vendor_product: ioctl (vendor) "
+	   "of device %d failed: %s\n", fd, strerror (errno));
+    if (ioctl (fd, B_SCANNER_IOCTL_PRODUCT, &product) != B_OK)
+      DBG (3, "sanei_usb_get_vendor_product: ioctl (product) "
+	   "of device %d failed: %s\n", fd, strerror (errno));
+    /* copy from 16 to 32 bit value */
+    *vendorID = vendor;
+    *productID = product;
+  }
+#endif /* defined (__linux__), defined(__BEOS__) */
+  /* put more os-dependant stuff ... */
 }
 
 void
@@ -206,8 +230,10 @@ sanei_usb_init (void)
 #if defined(__linux__)
     "/dev/", "usbscanner",
     "/dev/usb/", "scanner",
-#elif defined(__FreeBSD__)
+#elif defined(__FreeBSD__) || defined(__NetBSD__) || defined (__OpenBSD__)
     "/dev/", "uscanner",
+#elif defined(__BEOS__)
+    "/dev/scanner/usb/", "",
 #endif
     0, 0
   };
@@ -246,7 +272,8 @@ sanei_usb_init (void)
 
       if (stat (dir_name, &stat_buf) < 0)
 	{
-	  DBG (5, "sanei_usb_init: can't stat %s: %s\n", dir_name, strerror (errno));
+	  DBG (5, "sanei_usb_init: can't stat %s: %s\n", dir_name,
+	       strerror (errno));
 	  continue;
 	}
       if (!S_ISDIR (stat_buf.st_mode))
@@ -257,22 +284,32 @@ sanei_usb_init (void)
       if ((dir = opendir (dir_name)) == 0)
 	{
 	  DBG (5, "sanei_usb_init: cannot read directory %s: %s\n", dir_name,
-		    strerror (errno));
+	       strerror (errno));
 	  continue;
 	}
-      
+
       while ((dir_entry = readdir (dir)) != 0)
 	{
+	  /* skip standard dir entries */
+	  if (strcmp (dir_entry->d_name, ".") == 0 || strcmp (dir_entry->d_name, "..") == 0)
+	  	continue;
+	  		
 	  if (strncmp (base_name, dir_entry->d_name, strlen (base_name)) == 0)
 	    {
-	      if (strlen (dir_name) + strlen (dir_entry->d_name) + 1 > sizeof (devname))
-		  continue;
+	      if (strlen (dir_name) + strlen (dir_entry->d_name) + 1 >
+		  sizeof (devname))
+		continue;
 	      sprintf (devname, "%s%s", dir_name, dir_entry->d_name);
-	      fd = open (devname, O_RDWR);
+	      fd = -1;
+#ifdef HAVE_RESMGR
+	      fd = rsm_open_device (devname, O_RDWR);
+#endif
+	      if (fd == -1)
+		fd = open (devname, O_RDWR);
 	      if (fd < 0)
 		{
 		  DBG (5, "sanei_usb_init: couldn't open %s: %s\n", devname,
-		      strerror (errno));
+		       strerror (errno));
 		  continue;
 		}
 	      vendor = -1;
@@ -281,18 +318,26 @@ sanei_usb_init (void)
 	      close (fd);
 	      devices[dn].devname = strdup (devname);
 	      if (!devices[dn].devname)
-		return;
+		{
+		  closedir (dir);
+		  return;
+		}
 	      devices[dn].vendor = vendor;
 	      devices[dn].product = product;
 	      devices[dn].method = sanei_usb_method_scanner_driver;
 	      devices[dn].open = SANE_FALSE;
-	      DBG (4, "sanei_usb_init: found kernel scanner device (0x%04x/0x%04x) at %s\n",
+	      DBG (4,
+		   "sanei_usb_init: found kernel scanner device (0x%04x/0x%04x) at %s\n",
 		   vendor, product, devname);
 	      dn++;
 	      if (dn >= MAX_DEVICES)
-		return;
+		{
+		  closedir (dir);
+		  return;
+		}
 	    }
 	}
+      closedir (dir);
     }
 
   /* Check for devices using libusb */
@@ -318,18 +363,22 @@ sanei_usb_init (void)
 
 	  if (!dev->config)
 	    {
-	      DBG (1, "sanei_usb_init: device 0x%04x/0x%04x is not configured\n",
+	      DBG (1,
+		   "sanei_usb_init: device 0x%04x/0x%04x is not configured\n",
 		   dev->descriptor.idVendor, dev->descriptor.idProduct);
 	      continue;
 	    }
 	  if (dev->descriptor.idVendor == 0 || dev->descriptor.idProduct == 0)
 	    {
-	      DBG (5, "sanei_usb_init: device 0x%04x/0x%04x looks like a root hub\n",
+	      DBG (5,
+		   "sanei_usb_init: device 0x%04x/0x%04x looks like a root hub\n",
 		   dev->descriptor.idVendor, dev->descriptor.idProduct);
 	      continue;
 	    }
 	  found = SANE_FALSE;
-	  for (interface = 0; interface < dev->config[0].bNumInterfaces && !found; interface++)
+	  for (interface = 0;
+	       interface < dev->config[0].bNumInterfaces && !found;
+	       interface++)
 	    {
 	      switch (dev->descriptor.bDeviceClass)
 		{
@@ -337,30 +386,35 @@ sanei_usb_init (void)
 		  found = SANE_TRUE;
 		  break;
 		case USB_CLASS_PER_INTERFACE:
-		  switch (dev->config[0].interface[interface].altsetting[0].bInterfaceClass)
+		  switch (dev->config[0].interface[interface].altsetting[0].
+			  bInterfaceClass)
 		    {
 		    case USB_CLASS_VENDOR_SPEC:
 		    case USB_CLASS_PER_INTERFACE:
-		    case 16:                /* data? */
+		    case 16:	/* data? */
 		      found = SANE_TRUE;
 		      break;
 		    }
 		  break;
 		}
 	      if (!found)
-		DBG (5, "sanei_usb_init: device 0x%04x/0x%04x, interface %d doesn't look like a "
+		DBG (5,
+		     "sanei_usb_init: device 0x%04x/0x%04x, interface %d doesn't look like a "
 		     "scanner (%d/%d)\n", dev->descriptor.idVendor,
-		     dev->descriptor.idProduct, interface, dev->descriptor.bDeviceClass, 
-		     dev->config[0].interface[interface].altsetting[0].bInterfaceClass);
+		     dev->descriptor.idProduct, interface,
+		     dev->descriptor.bDeviceClass,
+		     dev->config[0].interface[interface].altsetting[0].
+		     bInterfaceClass);
 	    }
 	  interface--;
 	  if (!found)
 	    {
-	      DBG (5, "sanei_usb_init: device 0x%04x/0x%04x: no suitable interfaces\n",
+	      DBG (5,
+		   "sanei_usb_init: device 0x%04x/0x%04x: no suitable interfaces\n",
 		   dev->descriptor.idVendor, dev->descriptor.idProduct);
 	      continue;
 	    }
-	  
+
 	  devices[dn].libusb_device = dev;
 	  snprintf (devname, sizeof (devname), "libusb:%s:%s",
 		    dev->bus->dirname, dev->filename);
@@ -372,8 +426,10 @@ sanei_usb_init (void)
 	  devices[dn].method = sanei_usb_method_libusb;
 	  devices[dn].open = SANE_FALSE;
 	  devices[dn].interface_nr = interface;
-	  DBG (4, "sanei_usb_init: found libusb device (0x%04x/0x%04x) interface %d  at %s\n",
-	       dev->descriptor.idVendor, dev->descriptor.idProduct, interface, devname);
+	  DBG (4,
+	       "sanei_usb_init: found libusb device (0x%04x/0x%04x) interface %d  at %s\n",
+	       dev->descriptor.idVendor, dev->descriptor.idProduct, interface,
+	       devname);
 	  dn++;
 	  if (dn >= MAX_DEVICES)
 	    return;
@@ -503,6 +559,7 @@ sanei_usb_open (SANE_String_Const devname, SANE_Int * dn)
 {
   int devcount;
   SANE_Bool found = SANE_FALSE;
+  int c, i, a;
 
   DBG (5, "sanei_usb_open: trying to open device `%s'\n", devname);
   if (!dn)
@@ -511,7 +568,8 @@ sanei_usb_open (SANE_String_Const devname, SANE_Int * dn)
       return SANE_STATUS_INVAL;
     }
 
-  for (devcount = 0; devcount < MAX_DEVICES && devices[devcount].devname != 0; devcount++)
+  for (devcount = 0; devcount < MAX_DEVICES && devices[devcount].devname != 0;
+       devcount++)
     {
       if (strcmp (devices[devcount].devname, devname) == 0)
 	{
@@ -538,7 +596,8 @@ sanei_usb_open (SANE_String_Const devname, SANE_Int * dn)
       struct usb_interface_descriptor *interface;
       int result, num;
 
-      devices[devcount].libusb_handle = usb_open (devices[devcount].libusb_device);
+      devices[devcount].libusb_handle =
+	usb_open (devices[devcount].libusb_device);
       if (!devices[devcount].libusb_handle)
 	{
 	  SANE_Status status = SANE_STATUS_INVAL;
@@ -572,7 +631,7 @@ sanei_usb_open (SANE_String_Const devname, SANE_Int * dn)
 	{
 	  DBG (3, "sanei_usb_open: more than one "
 	       "configuration (%d), choosing first config (%d)\n",
-	       dev->descriptor.bNumConfigurations, 
+	       dev->descriptor.bNumConfigurations,
 	       dev->config[0].bConfigurationValue);
 	}
       result = usb_set_configuration (devices[devcount].libusb_handle,
@@ -597,9 +656,9 @@ sanei_usb_open (SANE_String_Const devname, SANE_Int * dn)
 	  usb_close (devices[devcount].libusb_handle);
 	  return status;
 	}
-      
+
       /* Claim the interface */
-      result = usb_claim_interface (devices[devcount].libusb_handle, 
+      result = usb_claim_interface (devices[devcount].libusb_handle,
 				    devices[devcount].interface_nr);
       if (result < 0)
 	{
@@ -621,73 +680,164 @@ sanei_usb_open (SANE_String_Const devname, SANE_Int * dn)
 	  usb_close (devices[devcount].libusb_handle);
 	  return status;
 	}
-      interface = &dev->config[0].interface->altsetting[0];
 
-      /* Now we look for usable endpoints */
-      for (num = 0; num < interface->bNumEndpoints; num++)
+      /* Loop through all of the configurations */
+      for (c = 0; c < dev->descriptor.bNumConfigurations; c++)
 	{
-	  struct usb_endpoint_descriptor *endpoint;
-	  int address, direction, transfer_type;
-
-	  endpoint = &interface->endpoint[num];
-	  address = endpoint->bEndpointAddress & USB_ENDPOINT_ADDRESS_MASK;
-	  direction = endpoint->bEndpointAddress & USB_ENDPOINT_DIR_MASK;
-	  transfer_type = endpoint->bmAttributes & USB_ENDPOINT_TYPE_MASK;
-
-	  /* save the endpoints we need later */
-	  if (transfer_type == USB_ENDPOINT_TYPE_INTERRUPT)
-	  {
-	    DBG (5, "sanei_usb_open: found interupt-%s endpoint (address %d)\n",
-	         direction ? "in" : "out", address);
-	    if (direction)	/* in */
+	  /* Loop through all of the interfaces */
+	  for (i = 0; i < dev->config[c].bNumInterfaces; i++)
 	    {
-	      if (devices[devcount].int_in_ep)
-		DBG (3, "sanei_usb_open: we already have a int-in endpoint "
-		     "(address: %d), ignoring the new one\n",
-		     devices[devcount].int_in_ep);
-	      else
-		devices[devcount].int_in_ep = endpoint->bEndpointAddress;
-	    }
-	    else
-	      if (devices[devcount].int_out_ep)
-		DBG (3, "sanei_usb_open: we already have a int-out endpoint "
-		     "(address: %d), ignoring the new one\n",
-		     devices[devcount].int_out_ep);
-	      else
-		devices[devcount].int_out_ep = endpoint->bEndpointAddress;
-	  }
-	  else if (transfer_type == USB_ENDPOINT_TYPE_BULK)
-	  {
-	    DBG (5, "sanei_usb_open: found bulk-%s endpoint (address %d)\n",
-	         direction ? "in" : "out", address);
-	    if (direction)	/* in */
-	      {
-		if (devices[devcount].bulk_in_ep)
-		  DBG (3, "sanei_usb_open: we already have a bulk-in endpoint "
-		       "(address: %d), ignoring the new one\n",
-		       devices[devcount].bulk_in_ep);
-		else
-		  devices[devcount].bulk_in_ep = endpoint->bEndpointAddress;
-	      }
-	    else
-	      {
-	        if (devices[devcount].bulk_out_ep)
-		  DBG (3, "sanei_usb_open: we already have a bulk-out endpoint "
-		       "(address: %d), ignoring the new one\n",
-		       devices[devcount].bulk_out_ep);
-	        else
-		  devices[devcount].bulk_out_ep = endpoint->bEndpointAddress;
-	      }
-	    }
-	  /* ignore currently unsupported endpoints */
-	  else {
-	      DBG (5, "sanei_usb_open: ignoring %s-%s endpoint "
-		   "(address: %d)\n",
-		   transfer_type == USB_ENDPOINT_TYPE_CONTROL ? "control" :
-		   transfer_type == USB_ENDPOINT_TYPE_ISOCHRONOUS
-		   ? "isochronous" : "interrupt",
-		   direction ? "in" : "out", address);
-	      continue;
+	      /* Loop through all of the alternate settings */
+	      for (a = 0; a < dev->config[c].interface[i].num_altsetting; a++)
+		{
+
+		  DBG (5, "sanei_usb_open: configuration nr: %d\n", c);
+		  DBG (5, "sanei_usb_open:     interface nr: %d\n", i);
+		  DBG (5, "sanei_usb_open:   alt_setting nr: %d\n", a);
+
+		  interface = &dev->config[c].interface[i].altsetting[a];
+
+		  /* Now we look for usable endpoints */
+		  for (num = 0; num < interface->bNumEndpoints; num++)
+		    {
+		      struct usb_endpoint_descriptor *endpoint;
+		      int address, direction, transfer_type;
+
+		      endpoint = &interface->endpoint[num];
+		      DBG (5, "sanei_usb_open:num: %d endpoint: 0x%x\n", num,
+			   endpoint);
+		      transfer_type =
+			endpoint->bmAttributes & USB_ENDPOINT_TYPE_MASK;
+		      address =
+			endpoint->
+			bEndpointAddress & USB_ENDPOINT_ADDRESS_MASK;
+		      direction =
+			endpoint->bEndpointAddress & USB_ENDPOINT_DIR_MASK;
+
+		      DBG (5, "sanei_usb_open: direction: %d)\n", direction);
+		      transfer_type =
+			endpoint->bmAttributes & USB_ENDPOINT_TYPE_MASK;
+
+		      DBG (5,
+			   "sanei_usb_open:  address: %d transfertype: %d\n",
+			   address, transfer_type);
+
+
+		      /* save the endpoints we need later */
+		      if (transfer_type == USB_ENDPOINT_TYPE_INTERRUPT)
+			{
+			  DBG (5,
+			       "sanei_usb_open: found interrupt-%s endpoint (address 0x%02x)\n",
+			       direction ? "in" : "out", address);
+			  if (direction)	/* in */
+			    {
+			      if (devices[devcount].int_in_ep)
+				DBG (3,
+				     "sanei_usb_open: we already have a int-in endpoint "
+				     "(address: 0x%02x), ignoring the new one\n",
+				     devices[devcount].int_in_ep);
+			      else
+				devices[devcount].int_in_ep =
+				  endpoint->bEndpointAddress;
+			    }
+			  else
+			    {
+			      if (devices[devcount].int_out_ep)
+				DBG (3,
+				     "sanei_usb_open: we already have a int-out endpoint "
+				     "(address: 0x%02x), ignoring the new one\n",
+				     devices[devcount].int_out_ep);
+			      else
+				devices[devcount].int_out_ep =
+				  endpoint->bEndpointAddress;
+			    }
+			}
+		      else if (transfer_type == USB_ENDPOINT_TYPE_BULK)
+			{
+			  DBG (5,
+			       "sanei_usb_open: found bulk-%s endpoint (address 0x%02x)\n",
+			       direction ? "in" : "out", address);
+			  if (direction)	/* in */
+			    {
+			      if (devices[devcount].bulk_in_ep)
+				DBG (3,
+				     "sanei_usb_open: we already have a bulk-in endpoint "
+				     "(address: 0x%02x), ignoring the new one\n",
+				     devices[devcount].bulk_in_ep);
+			      else
+				devices[devcount].bulk_in_ep =
+				  endpoint->bEndpointAddress;
+			    }
+			  else
+			    {
+			      if (devices[devcount].bulk_out_ep)
+				DBG (3,
+				     "sanei_usb_open: we already have a bulk-out endpoint "
+				     "(address: 0x%02x), ignoring the new one\n",
+				     devices[devcount].bulk_out_ep);
+			      else
+				devices[devcount].bulk_out_ep =
+				  endpoint->bEndpointAddress;
+			    }
+			}
+		      else if (transfer_type == USB_ENDPOINT_TYPE_ISOCHRONOUS)
+			{
+			  DBG (5,
+			       "sanei_usb_open: found isochronous-%s endpoint (address 0x%02x)\n",
+			       direction ? "in" : "out", address);
+			  if (direction)	/* in */
+			    {
+			      if (devices[devcount].iso_in_ep)
+				DBG (3,
+				     "sanei_usb_open: we already have a isochronous-in endpoint "
+				     "(address: 0x%02x), ignoring the new one\n",
+				     devices[devcount].iso_in_ep);
+			      else
+				devices[devcount].iso_in_ep =
+				  endpoint->bEndpointAddress;
+			    }
+			  else
+			    {
+			      if (devices[devcount].iso_out_ep)
+				DBG (3,
+				     "sanei_usb_open: we already have a isochronous-out endpoint "
+				     "(address: 0x%02x), ignoring the new one\n",
+				     devices[devcount].iso_out_ep);
+			      else
+				devices[devcount].iso_out_ep =
+				  endpoint->bEndpointAddress;
+			    }
+			}
+		      else if (transfer_type == USB_ENDPOINT_TYPE_CONTROL)
+			{
+			  DBG (5,
+			       "sanei_usb_open: found control-%s endpoint (address 0x%02x)\n",
+			       direction ? "in" : "out", address);
+			  if (direction)	/* in */
+			    {
+			      if (devices[devcount].control_in_ep)
+				DBG (3,
+				     "sanei_usb_open: we already have a control-in endpoint "
+				     "(address: 0x%02x), ignoring the new one\n",
+				     devices[devcount].control_in_ep);
+			      else
+				devices[devcount].control_in_ep =
+				  endpoint->bEndpointAddress;
+			    }
+			  else
+			    {
+			      if (devices[devcount].control_out_ep)
+				DBG (3,
+				     "sanei_usb_open: we already have a control-out endpoint "
+				     "(address: 0x%02x), ignoring the new one\n",
+				     devices[devcount].control_out_ep);
+			      else
+				devices[devcount].control_out_ep =
+				  endpoint->bEndpointAddress;
+			    }
+			}
+		    }
+		}
 	    }
 	}
 #else /* not HAVE_LIBUSB */
@@ -700,7 +850,12 @@ sanei_usb_open (SANE_String_Const devname, SANE_Int * dn)
     {
       long int flag;
       /* Using kernel scanner driver */
-      devices[devcount].fd = open (devname, O_RDWR);
+      devices[devcount].fd = -1;
+#ifdef HAVE_RESMGR
+      devices[devcount].fd = rsm_open_device (devname, O_RDWR);
+#endif
+      if (devices[devcount].fd == -1)
+	devices[devcount].fd = open (devname, O_RDWR);
       if (devices[devcount].fd < 0)
 	{
 	  SANE_Status status = SANE_STATUS_INVAL;
@@ -727,7 +882,8 @@ sanei_usb_open (SANE_String_Const devname, SANE_Int * dn)
     }
   else
     {
-      DBG (1, "sanei_usb_open: access method %d not implemented\n", devices[devcount].method);
+      DBG (1, "sanei_usb_open: access method %d not implemented\n",
+	   devices[devcount].method);
       return SANE_STATUS_INVAL;
     }
 
@@ -762,11 +918,13 @@ sanei_usb_close (SANE_Int dn)
       /* Should only be done in case of a stall */
       usb_clear_halt (devices[dn].libusb_handle, devices[dn].bulk_in_ep);
       usb_clear_halt (devices[dn].libusb_handle, devices[dn].bulk_out_ep);
+      usb_clear_halt (devices[dn].libusb_handle, devices[dn].iso_in_ep);
       /* be careful, we don't know if we are in DATA0 stage now */
-      usb_resetep(devices[dn].libusb_handle, devices[dn].bulk_in_ep);
-      usb_resetep(devices[dn].libusb_handle, devices[dn].bulk_out_ep);
+      usb_resetep (devices[dn].libusb_handle, devices[dn].bulk_in_ep);
+      usb_resetep (devices[dn].libusb_handle, devices[dn].bulk_out_ep);
+      usb_resetep (devices[dn].libusb_handle, devices[dn].iso_in_ep);
 #endif
-      usb_release_interface (devices[dn].libusb_handle, 
+      usb_release_interface (devices[dn].libusb_handle,
 			     devices[dn].interface_nr);
       usb_close (devices[dn].libusb_handle);
     }
@@ -777,6 +935,16 @@ sanei_usb_close (SANE_Int dn)
   return;
 }
 
+void
+sanei_usb_set_timeout (SANE_Int timeout)
+{
+#ifdef HAVE_LIBUSB
+  libusb_timeout = timeout;
+#else
+  DBG (1, "sanei_usb_close: libusb support missing\n");
+#endif /* HAVE_LIBUSB */
+}
+
 SANE_Status
 sanei_usb_read_bulk (SANE_Int dn, SANE_Byte * buffer, size_t * size)
 {
@@ -846,7 +1014,7 @@ sanei_usb_read_bulk (SANE_Int dn, SANE_Byte * buffer, size_t * size)
   DBG (5, "sanei_usb_read_bulk: wanted %lu bytes, got %ld bytes\n",
        (unsigned long) *size, (unsigned long) read_size);
   *size = read_size;
-    
+
   return SANE_STATUS_GOOD;
 }
 
@@ -896,7 +1064,7 @@ sanei_usb_write_bulk (SANE_Int dn, const SANE_Byte * buffer, size_t * size)
 #endif /* not HAVE_LIBUSB */
   else
     {
-      DBG (1, "sanei_usb_read_bulk: access method %d not implemented\n",
+      DBG (1, "sanei_usb_write_bulk: access method %d not implemented\n",
 	   devices[dn].method);
       return SANE_STATUS_INVAL;
     }
@@ -924,7 +1092,8 @@ sanei_usb_control_msg (SANE_Int dn, SANE_Int rtype, SANE_Int req,
 {
   if (dn >= MAX_DEVICES || dn < 0)
     {
-      DBG (1, "sanei_usb_control_msg: dn >= MAX_DEVICES || dn < 0\n");
+      DBG (1, "sanei_usb_control_msg: dn >= MAX_DEVICES || dn < 0, dn=%d\n",
+	   dn);
       return SANE_STATUS_INVAL;
     }
 
@@ -954,6 +1123,26 @@ sanei_usb_control_msg (SANE_Int dn, SANE_Int rtype, SANE_Int req,
       if ((rtype & 0x80) && debug_level > 10)
 	print_buffer (data, len);
       return SANE_STATUS_GOOD;
+#elif defined(__BEOS__)
+      struct usb_scanner_ioctl_ctrlmsg c;
+
+      c.req.request_type = rtype;
+      c.req.request = req;
+      c.req.value = value;
+      c.req.index = index;
+      c.req.length = len;
+      c.data = data;
+
+      if (ioctl (devices[dn].fd, B_SCANNER_IOCTL_CTRLMSG, &c) < 0)
+	{
+	  DBG (5, "sanei_usb_control_msg: SCANNER_IOCTL_CTRLMSG error - %s\n",
+	       strerror (errno));
+	  return SANE_STATUS_IO_ERROR;
+	}
+	if ((rtype & 0x80) && debug_level > 10)
+		print_buffer (data, len);
+	
+	return SANE_STATUS_GOOD;
 #else /* not __linux__ */
       DBG (5, "sanei_usb_control_msg: not supported on this OS\n");
       return SANE_STATUS_UNSUPPORTED;
@@ -1007,18 +1196,23 @@ sanei_usb_read_int (SANE_Int dn, SANE_Byte * buffer, size_t * size)
       DBG (1, "sanei_usb_read_int: dn >= MAX_DEVICES || dn < 0\n");
       return SANE_STATUS_INVAL;
     }
-  if (devices[dn].method == sanei_usb_method_scanner_driver) {
-     DBG (1, "sanei_usb_read_int: access method %d not implemented\n",
+
+  DBG (5, "sanei_usb_read_int: trying to read %lu bytes\n",
+       (unsigned long) *size);
+  if (devices[dn].method == sanei_usb_method_scanner_driver)
+    {
+      DBG (1, "sanei_usb_read_int: access method %d not implemented\n",
 	   devices[dn].method);
-     return SANE_STATUS_INVAL;
-  }
+      return SANE_STATUS_INVAL;
+    }
   else if (devices[dn].method == sanei_usb_method_libusb)
 #ifdef HAVE_LIBUSB
     {
       if (devices[dn].int_in_ep)
-	read_size = usb_bulk_read (devices[dn].libusb_handle,
-				   devices[dn].int_in_ep, (char *) buffer,
-				   (int) *size, libusb_timeout);
+	read_size = usb_interrupt_read (devices[dn].libusb_handle,
+					devices[dn].int_in_ep,
+					(char *) buffer, (int) *size,
+					libusb_timeout);
       else
 	{
 	  DBG (1, "sanei_usb_read_int: can't read without an int "
@@ -1060,5 +1254,210 @@ sanei_usb_read_int (SANE_Int dn, SANE_Byte * buffer, size_t * size)
   *size = read_size;
   if (debug_level > 10)
     print_buffer (buffer, read_size);
+
   return SANE_STATUS_GOOD;
 }
+
+SANE_Status
+sanei_usb_set_configuration (SANE_Int dn, SANE_Int configuration)
+{
+  if (dn >= MAX_DEVICES || dn < 0)
+    {
+      DBG (1,
+	   "sanei_usb_set_configuration: dn >= MAX_DEVICES || dn < 0, dn=%d\n",
+	   dn);
+      return SANE_STATUS_INVAL;
+    }
+
+  DBG (5, "sanei_usb_set_configuration: configuration = %d\n", configuration);
+
+  if (devices[dn].method == sanei_usb_method_scanner_driver)
+    {
+#if defined(__linux__)
+      return SANE_STATUS_GOOD;
+#else /* not __linux__ */
+      DBG (5, "sanei_usb_set_configuration: not supported on this OS\n");
+      return SANE_STATUS_UNSUPPORTED;
+#endif /* not __linux__ */
+    }
+  else if (devices[dn].method == sanei_usb_method_libusb)
+#ifdef HAVE_LIBUSB
+    {
+      int result;
+
+      result =
+	usb_set_configuration (devices[dn].libusb_handle, configuration);
+      if (result < 0)
+	{
+	  DBG (1, "sanei_usb_set_configuration: libusb complained: %s\n",
+	       usb_strerror ());
+	  return SANE_STATUS_INVAL;
+	}
+      return SANE_STATUS_GOOD;
+    }
+#else /* not HAVE_LIBUSB */
+    {
+      DBG (1, "sanei_usb_set_configuration: libusb support missing\n");
+      return SANE_STATUS_UNSUPPORTED;
+    }
+#endif /* not HAVE_LIBUSB */
+  else
+    {
+      DBG (1,
+	   "sanei_usb_set_configuration: access method %d not implemented\n",
+	   devices[dn].method);
+      return SANE_STATUS_UNSUPPORTED;
+    }
+}
+
+SANE_Status
+sanei_usb_claim_interface (SANE_Int dn, SANE_Int interface)
+{
+  if (dn >= MAX_DEVICES || dn < 0)
+    {
+      DBG (1,
+	   "sanei_usb_claim_interface: dn >= MAX_DEVICES || dn < 0, dn=%d\n",
+	   dn);
+      return SANE_STATUS_INVAL;
+    }
+
+  DBG (5, "sanei_usb_claim_interface: interface = %d\n", interface);
+
+  if (devices[dn].method == sanei_usb_method_scanner_driver)
+    {
+#if defined(__linux__)
+      return SANE_STATUS_GOOD;
+#else /* not __linux__ */
+      DBG (5, "sanei_usb_claim_interface: not supported on this OS\n");
+      return SANE_STATUS_UNSUPPORTED;
+#endif /* not __linux__ */
+    }
+  else if (devices[dn].method == sanei_usb_method_libusb)
+#ifdef HAVE_LIBUSB
+    {
+      int result;
+
+      result = usb_claim_interface (devices[dn].libusb_handle, interface);
+      if (result < 0)
+	{
+	  DBG (1, "sanei_usb_claim_interface: libusb complained: %s\n",
+	       usb_strerror ());
+	  return SANE_STATUS_INVAL;
+	}
+      return SANE_STATUS_GOOD;
+    }
+#else /* not HAVE_LIBUSB */
+    {
+      DBG (1, "sanei_usb_claim_interface: libusb support missing\n");
+      return SANE_STATUS_UNSUPPORTED;
+    }
+#endif /* not HAVE_LIBUSB */
+  else
+    {
+      DBG (1, "sanei_usb_claim_interface: access method %d not implemented\n",
+	   devices[dn].method);
+      return SANE_STATUS_UNSUPPORTED;
+    }
+}
+
+SANE_Status
+sanei_usb_release_interface (SANE_Int dn, SANE_Int interface)
+{
+  if (dn >= MAX_DEVICES || dn < 0)
+    {
+      DBG (1,
+	   "sanei_usb_release_interface: dn >= MAX_DEVICES || dn < 0, dn=%d\n",
+	   dn);
+      return SANE_STATUS_INVAL;
+    }
+
+  DBG (5, "sanei_usb_release_interface: interface = %d\n", interface);
+
+  if (devices[dn].method == sanei_usb_method_scanner_driver)
+    {
+#if defined(__linux__)
+      return SANE_STATUS_GOOD;
+#else /* not __linux__ */
+      DBG (5, "sanei_usb_release_interface: not supported on this OS\n");
+      return SANE_STATUS_UNSUPPORTED;
+#endif /* not __linux__ */
+    }
+  else if (devices[dn].method == sanei_usb_method_libusb)
+#ifdef HAVE_LIBUSB
+    {
+      int result;
+
+      result = usb_release_interface (devices[dn].libusb_handle, interface);
+      if (result < 0)
+	{
+	  DBG (1, "sanei_usb_release_interface: libusb complained: %s\n",
+	       usb_strerror ());
+	  return SANE_STATUS_INVAL;
+	}
+      return SANE_STATUS_GOOD;
+    }
+#else /* not HAVE_LIBUSB */
+    {
+      DBG (1, "sanei_usb_release_interface: libusb support missing\n");
+      return SANE_STATUS_UNSUPPORTED;
+    }
+#endif /* not HAVE_LIBUSB */
+  else
+    {
+      DBG (1,
+	   "sanei_usb_release_interface: access method %d not implemented\n",
+	   devices[dn].method);
+      return SANE_STATUS_UNSUPPORTED;
+    }
+}
+
+SANE_Status
+sanei_usb_set_altinterface (SANE_Int dn, SANE_Int alternate)
+{
+  if (dn >= MAX_DEVICES || dn < 0)
+    {
+      DBG (1,
+	   "sanei_usb_set_altinterface: dn >= MAX_DEVICES || dn < 0, dn=%d\n",
+	   dn);
+      return SANE_STATUS_INVAL;
+    }
+
+  DBG (5, "sanei_usb_set_altinterface: alternate = %d\n", alternate);
+
+  if (devices[dn].method == sanei_usb_method_scanner_driver)
+    {
+#if defined(__linux__)
+      return SANE_STATUS_GOOD;
+#else /* not __linux__ */
+      DBG (5, "sanei_usb_set_altinterface: not supported on this OS\n");
+      return SANE_STATUS_UNSUPPORTED;
+#endif /* not __linux__ */
+    }
+  else if (devices[dn].method == sanei_usb_method_libusb)
+#ifdef HAVE_LIBUSB
+    {
+      int result;
+
+      result = usb_set_altinterface (devices[dn].libusb_handle, alternate);
+      if (result < 0)
+	{
+	  DBG (1, "sanei_usb_set_altinterface: libusb complained: %s\n",
+	       usb_strerror ());
+	  return SANE_STATUS_INVAL;
+	}
+      return SANE_STATUS_GOOD;
+    }
+#else /* not HAVE_LIBUSB */
+    {
+      DBG (1, "sanei_set_altinterface: libusb support missing\n");
+      return SANE_STATUS_UNSUPPORTED;
+    }
+#endif /* not HAVE_LIBUSB */
+  else
+    {
+      DBG (1,
+	   "sanei_usb_set_altinterface: access method %d not implemented\n",
+	   devices[dn].method);
+      return SANE_STATUS_UNSUPPORTED;
+    }
+}
